/*
  Copyright 1999 Brian H. Trammell (brian@altara.org)
  Copyright 1999 Ben Parrott (benji@altara.org)
  
  This software (and its source code) are freely usable, modifiable, 
  and redistributable under the terms of the GNU General Public License, 
  available at http://www.gnu.org/copyleft/gpl.html, and in the ./COPYING 
  file in the MARS distribution.
*/

package org.altara.mars;

import java.util.zip.*;
import java.util.*;
import java.io.*;

/**
   The configuration for MARS. Holds a table of Hosts and provides access to
   them, as well as timeout values and other user preferences.
*/

// Revision History:
// Revision 0.1 08 Feb 1999 bht - initial entry
// Revision 0.2 25 Feb 1999 bht - added support for deleting numbered hosts
//                              - added cloning support
// Revision 1.2.0 21 Mar 1999 bht - fixed deep cloning bug
// Revision 1.3.2 12 Apr 1999 bht - added move host up/down list function
// Revision 1.4.1 29 Apr 1999 bht - added support for SMTP Server/Note Address
// Revision 1.4.4 11 May 1999 bht - fixed the Bowers bug. May have
//                                  brought back the deep cloning bug, though.

public class MARSConfig implements Serializable, Cloneable {
    
    // Instance variables
    Vector hostTable;                    // All the Hosts MARS knows about
    long timeout;                        // Probe/SPOTSQuery timeout
    long autowait;                       // ProbeThread wait time
    float maxload;                       // Maximum load average before alert
    int mindisk;                         // Minimum free disk space
    String notifyServer;                 // SMTP server to use for notification
    String notifyAddr;                   // Address to notify on down service

    // Class variables
    static MARSConfig currentConfig;

    // Default value constants
    /** The default timeout value is 3000 ms. */
    public final static long defTimeout = 3000;
    /** The default thread wait time is 30000 ms. */
    public final static long defAutowait = 30000;
    /** The default maximum load average is 2.00 */
    public final static float defMaxload = (float)2.00;
    /** The minumum percent free disk space is 10% */
    public final static int defMindisk = 10;
    
    /** The name of the MARS configuration file. */
    public final static String marsConfigFile = "mars.conf";

    /**
       Create a new MARSConfig with an empty host list and default
       timeout delay values. This should only be called when MARS is started
       for the first time and cannot deserialize its MARSConfig.
    
       @return a new, clean MARSConfig
    */

    public MARSConfig() {
	hostTable = new Vector();
	timeout = defTimeout;
	autowait = defAutowait;
	maxload = defMaxload;
	mindisk = defMindisk;
	notifyServer = null;               // no notification by default
	notifyAddr = null;
    }

    /**
       Add a Host to the host table

       @param host the Host to add
    */
    
    public void addHost(Host host) {
	hostTable.addElement(host);
    }

    /**
       Delete a numbered Host from the table
       
       @param hostNum the number of the host to remove
    */
    
    public void deleteHostAt(int hostNum) {
	hostTable.removeElementAt(hostNum);
    }

    /**
       Delete a Host from the table
       
       @param host the Host to delete
       @return whether or not the host was deleted
    */
    
    public boolean deleteHost(Host host) {
	return hostTable.removeElement(host);
    }

    /**
       Move a numbered Host up the list
       
       @param hostNum the number of the element to move up
    */

    public void promoteHost(int hostNum) {
	
	// Can't promote the number one host
	if (hostNum <= 0)
	    return;
	
	// Swap the nth and (n-1)th element
	hostTable.insertElementAt(hostTable.elementAt(hostNum),
				  hostNum - 1);
	hostTable.removeElementAt(hostNum + 1);
    }

    /**
       Move a numbered Host down the list
       
       @param hostNum the number of the host to move down
    */

    public void demoteHost(int hostNum) {
	
	// Can't demote the last host
	if (hostNum >= (hostTable.size() - 1)) 
	    return;

	// Swap the nth and (n+1)th element
	hostTable.insertElementAt(hostTable.elementAt(hostNum),
				  hostNum + 2);
	hostTable.removeElementAt(hostNum);
    }

    /**
       Get an numbered Host

       @param hostNum the number of the element to get
       @return the specified Host
    */
    
    public Host getHost(int hostNum) {
	return ((Host)hostTable.elementAt(hostNum));
    }

    /**
       Get the number of hosts in the table

       @return the number of hosts MARS knows about
    */
    
    public int getHostCount() {
	return hostTable.size();
    }
    
    /** 
	Returns the current Probe/SPOTSQuery network timeout delay.
    
	@return the current timeout delay in milliseconds
    */
    
    public long getTimeout() {
	return timeout;
    }

    /** 
	Sets the Probe/SPOTSQuery network timeout delay.
    
	@param timeout the new timeout delay in milliseconds
    */
    
    public void setTimeout(long timeout) {
	this.timeout = timeout;
    }

    /** 
	Returns the current between-probe delay
	
	@return the current delay between probes in milliseconds
    */
    
    public long getAutowait() {
	return autowait;
    }
    
    /** 
	Sets the between-probe delay
	
	@param autowait the new delay between probes in milliseconds
    */
    
    public void setAutowait(long autowait) {
	this.autowait = autowait;
    }
    
    /** 
	Returns the current load average alert threshold
	
	@return the current maximum allowed load average
    */
    
    public float getMaxload() {
	return maxload;
    }
    
    /** 
	Sets the load average alert threshold
	
	@param maxload the new maximum allowed load average
    */
    
    public void setMaxload(float maxload) {
	this.maxload = maxload;
    }
    
    /** 
	Returns the current free disk space alert threshold
	
	@return the current minimum allowed free disk space
    */
    
    public int getMindisk() {
	return mindisk;
    }
    
    /** 
	Sets the disk space alert threshold
	
	@param mindisk the new minumum free disk space
    */
    
    public void setMindisk(int mindisk) {
	this.mindisk = mindisk;
    }
    
    /**
       Makes this instance of MARSConfig the current MARSConfig
    */

    public void makeCurrent() {
	currentConfig = this;
    }

    /** 
	Return a clone of this MARSConfig
    */
    public Object clone() {
	Host thisHost;
	try {
	    /* Create a clone of this configuration */
	    MARSConfig cloneConfig = (MARSConfig)super.clone();
	    /* Build a new Vector for deep cloning */
	    Vector cloneTable = new Vector();                   
	    /* Get an enumeration of the current host table */
	    Enumeration hostElements = hostTable.elements();
	    /* Deep copy the host table */
	    while (hostElements.hasMoreElements()) {
		/* get this host */
		thisHost = (Host)(hostElements.nextElement());
		/* clone it, put it in the new table */
		cloneTable.addElement(thisHost);
	    }
	    cloneConfig.hostTable = cloneTable;
	    return cloneConfig;
	} catch (CloneNotSupportedException ex) {
	    throw new InternalError();
	}
    }

    /**
       Return the current MARSConfig

       @return the currently active MARSConfig (as set by makeCurrent())
    */
    public static MARSConfig getCurrent() {
	return currentConfig;
    }

    /**
       Return a clone of the current MARSConfig

       @return an identical copy of the currentConfig
    */
    public static MARSConfig getClone() {
	return (MARSConfig)currentConfig.clone();
    }
    
    /**
       Serialize the current MARSConfig to the MARS configuration file.
    */

    public static void writeConfigFile() {
	try {
	    FileOutputStream fileOut = new FileOutputStream(marsConfigFile);
	    GZIPOutputStream zipFileOut = new GZIPOutputStream(fileOut);
	    ObjectOutputStream objOut = new ObjectOutputStream(zipFileOut);
	    objOut.writeObject(currentConfig);
	    objOut.flush();
	    objOut.close();
	} catch (Exception ex) {
	    System.err.println("MARS could not write config file "+marsConfigFile);
	    System.err.println(ex);
	}
    }

    /**
       Deserialize the configuration file to the current MARSConfig.
    
       @return true if the config file was read correctly, false if not.
    */
    public static boolean readConfigFile() {
	try {
	    FileInputStream fileIn = new FileInputStream(marsConfigFile);
	    GZIPInputStream zipFileIn = new GZIPInputStream(fileIn);
	    ObjectInputStream objIn = new ObjectInputStream(zipFileIn);
	    currentConfig = (MARSConfig)objIn.readObject();
	    objIn.close();
	} catch (Exception ex) {
	    return false;
	}
	return true;
    }

    /**
       Set up mail notification
     
       @param server SMTP server to use for notification services
       @param addr Address to notify on down alerts
    */
    public void setNotify(String server, String addr) {
	notifyServer = server;
	notifyAddr = addr;
    }

    /**
       Disable mail notification
    */
    public void clearNotify() {
	notifyServer = null;
	notifyAddr = null;
    }

    /**
       Return SMTP server to use for mail notification
       
       @return the name of the SMTP server
    */
    public String getNotifyServer() {
	return notifyServer;
    }

    /**
       Return address to send mail notification to
       
       @return the notification address
    */
    public String getNotifyAddr() {
	return notifyAddr;
    }
}







