/*
  Copyright 1999 Brian H. Trammell (brian@altara.org)
  Copyright 1999 Ben Parrott (benji@altara.org)
  
  This software (and its source code) are freely usable, modifiable, 
  and redistributable under the terms of the GNU General Public License, 
  available at http://www.gnu.org/copyleft/gpl.html, and in the ./COPYING 
  file in the MARS distribution.
*/

package org.altara.mars;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
   This class creates the config UI interface, and sets up action listners
   for User Input   
*/

// Revision 1.2.2: 25 Mar 1999 bht - added WindowAdapter to treat close box
//                                   like cancel button
// Revision 1.3.0: 06 Apr 1999 bht - gratuitously gutted the code because
//                                   I'm bored. No, really...
//                                   Removed toolbar, replaced with menubar.
//                                   Added stub support for promote/demote host
// Revision 1.3.2: 12 Apr 1999 bht - Added MoveUp/MoveDown actions.
// Revision 1.3.3: 15 Apr 1999 bht - Added Set Name / Set FQDN actions.
//                                   Added automatic nodename trim to Add Host.

public class MARSConfigUI extends JFrame
{

/** Width of the status checkboxes in the config window */
public static final int STATWIDTH = 45;
/** Width of the hostname column in the config window */
public static final int HOSTWIDTH = 180;

/* the Scroll Pane that allow for the user to see the entire table */
JScrollPane scrollPane;
/* Table model for configuration edits */
ConfigTableModel hostModel;    
/* Configuration host table */
JTable hostTable;
/* Keep the "current" config UI to keep the user from opening more than one */
static MARSConfigUI currentUI;
/* Disable Mail Notification item */
JMenuItem disMailItem;

/* Temporary holder for a "clone config" all changes are made to before the
   user chooses Save to commit changes. */
MARSConfig cloneConfig;

/**
 * Creates a new MARSConfigUI
 * 
 */
public MARSConfigUI()
{
  /* Sets the string at the top of the window (wish I knew how to get
   * rid of that damn coffee cup
   */
  super("MARS Configuration");

  /* Create the config UI menubar and associated menus
     (File, Host, Adjust) */
  JMenuBar configMenus = new JMenuBar();
  this.setJMenuBar(configMenus);

  JMenu fileMenu = new JMenu("File");
  JMenu hostMenu = new JMenu("Host");
  JMenu adjustMenu = new JMenu("Adjust");
  JMenu notifyMenu = new JMenu("Notification");

  /* Now, create new Actions and add them to menus with key accelerators */
  // the File menu
  MARSAction saveAction = new MARSAction(MARSAction.SAVE);
  fileMenu.add(saveAction).setAccelerator(
               KeyStroke.getKeyStroke('S',KeyEvent.CTRL_MASK));
  MARSAction cancelAction = new MARSAction(MARSAction.CANCEL);
  fileMenu.add(cancelAction).setAccelerator(
               KeyStroke.getKeyStroke('X',KeyEvent.CTRL_MASK));
  
  // the Host menu
  MARSAction addHostAction = new MARSAction(MARSAction.ADD);
  hostMenu.add(addHostAction).setAccelerator(
	       KeyStroke.getKeyStroke('A',KeyEvent.CTRL_MASK));
  MARSAction delHostAction = new MARSAction(MARSAction.DELETE);
  hostMenu.add(delHostAction).setAccelerator(
	       KeyStroke.getKeyStroke('D',KeyEvent.CTRL_MASK));
  hostMenu.addSeparator();
  MARSAction setVisNameAction = new MARSAction(MARSAction.SETVIS);
  hostMenu.add(setVisNameAction).setAccelerator(
	       KeyStroke.getKeyStroke('V',KeyEvent.CTRL_MASK));
  MARSAction setDnsNameAction = new MARSAction(MARSAction.SETDNS);
  hostMenu.add(setDnsNameAction).setAccelerator(
	       KeyStroke.getKeyStroke('F',KeyEvent.CTRL_MASK));
  hostMenu.addSeparator();
  MARSAction moveUpAction = new MARSAction(MARSAction.MOVEUP);
  hostMenu.add(moveUpAction).setAccelerator(
	       KeyStroke.getKeyStroke('Q',KeyEvent.CTRL_MASK));
  MARSAction moveDownAction = new MARSAction(MARSAction.MOVEDOWN);
  hostMenu.add(moveDownAction).setAccelerator(
	       KeyStroke.getKeyStroke('Z',KeyEvent.CTRL_MASK));
  
  // the Adjust menu
  MARSAction updateAction = new MARSAction(MARSAction.UPDATE);
  adjustMenu.add(updateAction).setAccelerator(
		 KeyStroke.getKeyStroke('U',KeyEvent.CTRL_MASK));
  MARSAction timeoutAction = new MARSAction(MARSAction.TIMEOUT);
  adjustMenu.add(timeoutAction).setAccelerator(
                 KeyStroke.getKeyStroke('T',KeyEvent.CTRL_MASK));
  adjustMenu.addSeparator();
  MARSAction loadAction = new MARSAction(MARSAction.MAXLOAD);
  adjustMenu.add(loadAction).setAccelerator(
		 KeyStroke.getKeyStroke('L',KeyEvent.CTRL_MASK));
  MARSAction diskAction = new MARSAction(MARSAction.MINDISK);
  adjustMenu.add(diskAction).setAccelerator(
		 KeyStroke.getKeyStroke('K',KeyEvent.CTRL_MASK));

  // the Notification menu
  MARSAction cfgMailAction = new MARSAction(MARSAction.CFGMAIL);
  notifyMenu.add(cfgMailAction);
  MARSAction disMailAction = new MARSAction(MARSAction.DISMAIL);
  disMailItem = notifyMenu.add(disMailAction);
  if (MARSConfig.getCurrent().getNotifyAddr() == null)
      disMailItem.setEnabled(false);
  else
      disMailItem.setEnabled(true);


  /* Add the menus to the menubar */
  configMenus.add(fileMenu);
  configMenus.add(hostMenu);
  configMenus.add(adjustMenu);
  configMenus.add(notifyMenu);

  /* Create a window listener to shutdown MARS when close box is clicked. */
  addWindowListener(new WindowAdapter() {
           public void windowClosing(WindowEvent ev) {
	       cancelPressed();
	   }});

  /* Clone the current configuration */
  cloneConfig = MARSConfig.getClone();

  /* Create a new host table model */
  hostModel = new ConfigTableModel(cloneConfig);
  /* Creates the hostTable from our Table Model */
  hostTable = new JTable(hostModel);

  /* Now set the cell widths of the columns */
  for (int i=0; i < MARS.SVC_COUNT; i++)
      hostTable.getColumn(MARS.SVC[i]).setPreferredWidth(STATWIDTH);
  hostTable.getColumn("SPOTS").setPreferredWidth(STATWIDTH);
  hostTable.getColumn("Host").setPreferredWidth(HOSTWIDTH);

  /* Add the table to the scroll pane */
  scrollPane = new JScrollPane(hostTable);

  /* Set layout manager to BorderLayout */
  getContentPane().setLayout(new BorderLayout());

  getContentPane().add(scrollPane,BorderLayout.CENTER);

  this.setSize(450,250);

  currentUI = this;
}

/**
   Used so other classes can access the current MARSConfigUI.
 */

public static MARSConfigUI getCurrent()
{
 return currentUI;
}

/**
   Handles Add Host menu item
 */
public void addPressed()
{
  String screenName = null;
  String FQDN = null;

  /* Ask the user for the FQDN and store it */
  FQDN = (String)JOptionPane.showInputDialog(this,
                "Enter the IP address or fully-qualified domain name\n"+
		"(host.domain.tld) of the new host:",
                "Add Host", JOptionPane.QUESTION_MESSAGE, null, null, "");
  /* If the user entered something without hitting cancel for the FQDN,
     find out how the FQDN should be displayed in the host table */
  if (FQDN != null)
  {
      String nodename = (new StringTokenizer(FQDN,".")).nextToken();
      
      screenName = (String)JOptionPane.showInputDialog(this,
            "Enter this host's name as it should appear in the host table:",
            "Add Host", JOptionPane.QUESTION_MESSAGE, null, null, nodename);
      
  
      /* Create a new host with no services enabled */
      Host newHost = new Host(FQDN);
      /* Set its visible name */
      if (screenName != null) {
	  newHost.setVisibleName(screenName);
      }
      /* Add it to the clone config's host table */
      cloneConfig.addHost(newHost);
      /* Tell the host table model to update itself */
      hostModel.fireTableDataChanged();
  }
}

/**
 * Handles when the user presses the Delete button
 */
public void deletePressed()
{
  /* Get the Host to be deleted */
  int hostToDelete = hostTable.getSelectedRow();
  
  /* Ignore delete if there is no selected host */
  if (hostToDelete < 0)
      return;

  /* Ask the user if it is okay to delete the host; results are stored in
     deleteOkay */
  int deleteOkay = JOptionPane.showConfirmDialog(this,
                "The selected host will be deleted from the system when\n"+
		"you save changes. Are you sure you want to delete?",
                "Confirm Delete", JOptionPane.YES_NO_OPTION,
                 JOptionPane.WARNING_MESSAGE);
  /* Delete the selected host if the user clicked OK */ 
  if (deleteOkay == JOptionPane.YES_OPTION) {
      cloneConfig.deleteHostAt(hostToDelete);
      /* Update the table */
      hostModel.fireTableDataChanged();
  }

}

/**
   Handles the Set Visible Name option
*/

public void setVisibleNamePressed() {
    /* Get the current visible name (for display) */
    int hostNumToRename = hostTable.getSelectedRow();
   
    /* ignore rename if no host selected */
    if (hostNumToRename < 0)
	return;

    Host hostToRename =cloneConfig.getHost(hostNumToRename);  
    String nodename = hostToRename.getVisibleName();

    /* Ask the user for a new visible name */
    nodename = (String)JOptionPane.showInputDialog(this,
      "Enter this host's name as it should appear in the host table:",
      "Set Visible Name", JOptionPane.QUESTION_MESSAGE, null, null, nodename);
    
    /* Set the visible name */
    if (nodename != null)
	hostToRename.setVisibleName(nodename);
    
    /* Update the display */
    hostModel.fireTableDataChanged();
    
}

/**
   Handles the Set DNS Name option
*/

public void setDnsNamePressed() {
    /* Get the current visible name (for display) */
    int hostNumToRename = hostTable.getSelectedRow();

    /* ignore rename if no host selected */
    if (hostNumToRename < 0)
	return;

    Host hostToRename =cloneConfig.getHost(hostNumToRename);  
    String FQDN = hostToRename.getDNSName();

    /* Ask the user for a new visible name */
    FQDN = (String)JOptionPane.showInputDialog(this,
      "Enter this host's fully-qualified domain name:",
      "Set FQDN", JOptionPane.QUESTION_MESSAGE, null, null, FQDN);
    
    /* Set the visible name */
    if (FQDN != null)
	hostToRename.setDNSName(FQDN);
    
    /* Update the display */
    hostModel.fireTableDataChanged();
    
}

/**
   Handles the Move Host Up option
*/

public void moveUpPressed() {
    /* Get the host to be promoted. */
    int hostToPromote = hostTable.getSelectedRow();
    
    /* Promote it */
    cloneConfig.promoteHost(hostToPromote);
    /* Update the table */
    hostModel.fireTableDataChanged();
}

/**
   Handles the Move Host Down option
*/

public void moveDownPressed() {
    /* Get the host to be promoted. */
    int hostToDemote = hostTable.getSelectedRow();
    
    /* Promote it */
    cloneConfig.demoteHost(hostToDemote);
    /* Update the table */
    hostModel.fireTableDataChanged();
}

/**
 * Handles when the user presses the Adjust Update Interval button
 */
public void adjustUpdateIntervalPressed()
{
  /* Get the current update interval (for display) */
  String currentInterval = String.valueOf(cloneConfig.getAutowait());  
  /* Store the UpdateInterval in a String based on the user to update */
  String UpdateInterval =
           (String)JOptionPane.showInputDialog(this,
                "Enter milliseconds to wait between updates",
                "Adjust Update Interval",
                JOptionPane.QUESTION_MESSAGE, null,
                null, currentInterval);
  /* Parse the input, set the clone config's autowait. */
  try {
      long updateMillis = Long.parseLong(UpdateInterval);
      cloneConfig.setAutowait(updateMillis);
  } catch (Exception ex) {
      /* should display an error here, but we don't yet. */
  }
}

/**
 * Handles when the user presses the Adjust Timeout button
 */
public void adjustTimeOutPressed()
{
  /* Get the current timeout interval (for display) */
  String currentTimeout = String.valueOf(cloneConfig.getTimeout());  
  /* Store the TimeOut period in a String based on the user input */
  String TimeOut =
           (String)JOptionPane.showInputDialog(this,
                "Enter milliseconds to wait for a service\n"+
	        "to respond before timing out",
                "Adjust Timeout Interval",
                JOptionPane.QUESTION_MESSAGE, null,
                null, currentTimeout);
  /* Parse the input, set the clone config's timeout. */
  try {
      long timeoutMillis = Long.parseLong(TimeOut);
      cloneConfig.setTimeout(timeoutMillis);
  } catch (Exception ex) {
      /* should display an error here, but we don't yet. */
  }
}

/**
 * Handles when the user presses the Adjust Load Threshold button
 */
public void adjustLoadPressed()
{
  /* Get the current load threshold (for display) */
  String currentMaxload = String.valueOf(cloneConfig.getMaxload());  
  /* Store the load threshold in a String based on the user input */
  String newMaxload =
           (String)JOptionPane.showInputDialog(this,
                "Enter maximum load average allowed\n"+
		"before MARS displays an alert",
                "New Load Average Threshold",
                JOptionPane.QUESTION_MESSAGE, null,
                null, currentMaxload);
  /* Parse the input, set the clone config's timeout. */
  try {
      float floatMaxload = Float.valueOf(newMaxload).floatValue();
      cloneConfig.setMaxload(floatMaxload);
  } catch (Exception ex) {
      /* should display an error here, but we don't yet. */
  }    
}

/**
 * Handles when the user presses the Adjust Disk Threshold button
 */
public void adjustDiskPressed()
{
  /* Get the current disk threshold (for display) */
  String currentMindisk = String.valueOf(cloneConfig.getMindisk());
  /* Store the disk threshold in a String based on the user input */
  String newMindisk =
           (String)JOptionPane.showInputDialog(this,
                "Enter minimum percentage of free disk space\n"+
		"allowed before MARS displays an alert.\n"+
	        "(Do not enter a % sign)",
                "New Free Disk Threshold",
                JOptionPane.QUESTION_MESSAGE, null,
                null, currentMindisk);
  /* Parse the input, set the clone config's timeout. */
  try {
      int intMindisk = Integer.parseInt(newMindisk);
      cloneConfig.setMindisk(intMindisk);
  } catch (Exception ex) {
      /* should display an error here, but we don't yet. */
  }    
}

/**
   Handles the Configure Mail Notification menu item
*/

public void configMailSelected() {
    String address = cloneConfig.getNotifyAddr();
    String server = cloneConfig.getNotifyServer();

    /* Ask the user for the notification address and store it */
    address = (String)JOptionPane.showInputDialog(this,
      "Enter the email address to notify of any alerts.",
      "Notification Address",
      JOptionPane.QUESTION_MESSAGE,null,null,address);

    if (address != null) {
	server = (String)JOptionPane.showInputDialog(this,
        "Enter the hostname of the SMTP server to use to send alert messages.",
        "Notification Server",
	JOptionPane.QUESTION_MESSAGE,null,null,server);
	
	if (server != null) {
	    
	    // We have all the data we need. Set up mail notification.
	    cloneConfig.setNotify(server,address);
	
	    // Enable the Disable Notification item
	    disMailItem.setEnabled(true);
	}
    }
}

/**
   Handles the Disable Mail Notification menu item
*/

public void disableMailSelected() {
    
    /* No confirmation. Just kill it. */
    cloneConfig.clearNotify();
    
    /* Disable this menu item */
    disMailItem.setEnabled(false);
}

/**
 * Handles when the user presses the Save button
 */
public void savePressed()
{
    // Saving is a big deal. Ask the user about it.
    int saveOkay = JOptionPane.showConfirmDialog(this,
                "All changes you have made since opening this config window\n"+
                "will be committed to disk and MARS will be restarted with\n"+
		"the new config. Are you sure you want to save changes?",
                "Confirm Save", JOptionPane.YES_NO_OPTION,
                 JOptionPane.WARNING_MESSAGE);
    if (saveOkay == JOptionPane.YES_OPTION) {

	// Stop MARS.
	MARS.stopThreads();
	// Make our cloned config the current one
	cloneConfig.makeCurrent();
	// Write the new config to disk
	MARSConfig.writeConfigFile();

     /* If we were running before, let's get running again */
     if ( MARSMainUI.getCurrent().returnRunningStatus() )
     {  // Okay. This is a bit tricky.
	// Clear the probe thread list
	ProbeThread.clearAll();
	// Reinitialize all the probe threads
	MARS.initProbes();
	// Restart all the threads
	MARS.startThreads();
     }
     
     MARSTableModel.updateCurrent();     // show config changes right away 
     
     currentUI = null;             // lose my reference
     this.dispose();               // close the window
    }
}

/**
 * Handles when the user presses the Cancel button
 */
public void cancelPressed()
{
    // All we have to do here is chuck the clone config and close the window.
    // be nice and ask the user first. :)
  int cancelOkay = JOptionPane.showConfirmDialog(this,
                "All changes you have made since opening this config window\n"+
                "will be lost.  Are you sure you want to cancel changes?",
                "Confirm Cancel", JOptionPane.YES_NO_OPTION,
                 JOptionPane.WARNING_MESSAGE);
  if (cancelOkay == JOptionPane.YES_OPTION) {
      cloneConfig = null;           // be gone, evil config!!
      currentUI = null;             // lose my reference
      this.dispose();               // close the window.
  }
}


} // end MARSConfigUI




