/*
  Copyright 1999 Brian H. Trammell (brian@altara.org)
  Copyright 1999 Ben Parrott (benji@altara.org)
  
  This software (and its source code) are freely usable, modifiable, 
  and redistributable under the terms of the GNU General Public License, 
  available at http://www.gnu.org/copyleft/gpl.html, and in the ./COPYING 
  file in the MARS distribution.
*/

package org.altara.mars;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
   This class creates the main UI interface, and sets up action and selection
   listeners to handle user input.
*/

// (c) 1999 Brian H. Trammell and Ben Parrott

// Revision 1.1: 12 Mar 1999 bht - added support for status bar, added
//                                 advanced table properties (color-coded
//                                 displays via TableRenderer)
// Revision 1.2.1: 21 Mar 1999 bht - added DisplayDownAlert() for new
//                                   alert semantics. changed background
//                                   color for timed out services.
// Revision 1.2.2: 25 Mar 1999 bht - cleaned up table renderer stuff
// Revision 1.2.5: 03 Apr 1999 bht - added support for ListSelectionListener,
//                                   changed color for ok services.
// Revision 1.3.0: 06 Apr 1999 bht - updated to reflect name change of
//                                   ButtonAction to MARSAction, and for new
//                                   MARSAction constants.
// Revision 1.3.3: 14 Apr 1999 bht - moved toolbar functions to menubar
// Revision 1.3.5: 23 Apr 1999 bht - updated display of N/A services.
// Revision 1.4.1: 26 Apr 1999 bht - added DisplayLastResponse and handler to
//                                   the ListSelectionListener
//                                 - added hooks to call MARSNotifier for
//                                   email notification
// Revision 1.4.2:  6 May 1999 bht - minor changes to statwidth/spotwidth
// Revision 1.4.4: 11 May 1999 bht - changed window width to fix "do.."
// Revision 1.4.6: 16 May 1999 bht - added initial support for per-filesystem
//                                   reporting. Major cleanup of method names
//                                   and documentation.
// Revision 1.4.7: 19 May 1999 bht - added DisplayDiskAlert method to display
//                                   per-filesystem disk space alerts.
// Revision 1.4.8:  7 Jun 1999 bht - fixed disk report NPE when SPOTS is down

public class MARSMainUI extends JFrame implements ListSelectionListener
{
    
    /** compile-time debug flag. Should be false */
    public final static boolean DEBUG = false;
    /** width of the status columns in the main table */
    public final static int STATWIDTH = 45;
    /** width of the SPOTS columns in the main table */
    public final static int SPOTWIDTH = 60;
    /** width of the hostname column in the main table */
    public final static int HOSTWIDTH = 180;
    
    /* i like to scroll */
    JScrollPane scrollPane;
    /* host model for the main display */
    MARSTableModel hostData;
    /* the main display */
    JTable hostTable;
    /* static reference to the current UI window. Used for warnings and status
       bars and the like */
    static MARSMainUI currentUI;
    /* the status bar. Whee! */
    JLabel statusBar;
    /* Tells us if we are in the run state or the stopped state */
    public boolean running = true;
    /* The stop menu item */
    JMenuItem stopItem;
    /* The start menu item */
    JMenuItem startItem;
    
    /**
       Create a new main window.
       The MARSMainUI will create the needed table based on the configuration,
       set up action listners for the buttons, and basically act as the user
       interface for MARS 
    */
    public MARSMainUI() {
	
	/* Sets the string at the top of the window (wish I knew how to get
	   rid of that damn coffee cup) */
	super("MARS "+MARS.VERSION);
	
	/* Create the config UI menubar and associated menus (File, Control) */
	JMenuBar mainMenus = new JMenuBar();
	this.setJMenuBar(mainMenus);
	
	JMenu fileMenu = new JMenu("File");
	JMenu controlMenu = new JMenu("Control");
	
	/* Now, create new Actions and add them to menus with accelerators */
	// the File menu
	MARSAction configAction = new MARSAction(MARSAction.CONFIGURE);
	fileMenu.add(configAction).setAccelerator(
		     KeyStroke.getKeyStroke('C',KeyEvent.CTRL_MASK));
	MARSAction quitAction = new MARSAction(MARSAction.QUIT);
	fileMenu.add(quitAction).setAccelerator(
	             KeyStroke.getKeyStroke('X',KeyEvent.CTRL_MASK));

	// the Control menu
	MARSAction startAction = new MARSAction(MARSAction.START);
	startItem = controlMenu.add(startAction);
	startItem.setAccelerator(
	      KeyStroke.getKeyStroke('S',KeyEvent.CTRL_MASK));
	startItem.setEnabled(false);
	MARSAction stopAction = new MARSAction(MARSAction.STOP);
	stopItem = controlMenu.add(stopAction);
	stopItem.setAccelerator(
	      KeyStroke.getKeyStroke('T',KeyEvent.CTRL_MASK));
  
	/* Add the menus to the menubar */
	mainMenus.add(fileMenu);
	mainMenus.add(controlMenu);
	
	/* Create a window listener to quit MARS when close box is clicked. */
	addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent ev) {
		MARS.shutdown();
	    }});

	/* Get a new TableModel */
	hostData = new MARSTableModel();
	
	/* Create the hostTable from our Table Model */
	hostTable = new JTable(hostData);
	
	/* here comes the heavy table wizardry. */
	/* Make the table cell backgrounds change color based on their status.
	   Red means down, yellow timed out, green OK, grey means not
	   available. */
	/* Start with a renderer that can change color based on value */
	DefaultTableCellRenderer statusColumnRenderer =
	    new DefaultTableCellRenderer() {
		public void setValue(Object value) {
		    /* Possibly pointless validation. Make sure the cell is a
		       string. If it's not a string, everything's gone right
		       to hell. :) */
		    String cellValue = (value instanceof String) ? 
		                       (String)value : "";
		    if (cellValue.equals(Host.TO))        // timeouts yellow
		    setBackground(Color.yellow);
		    else if (cellValue.equals(Host.FAIL)) // failures are red
		    setBackground(Color.red);
		    else if (cellValue.equals(Host.OK))   // ok is green
		    setBackground(Color.green);
		    else 
		    setBackground(Color.lightGray);   // grey out N/A services
		    setText(cellValue);                   // now pass the value
		}
	    };
  
	/* Now get each status column and set its cell renderer and width */
	for (int i=0; i < MARS.SVC_COUNT; i++) {
	    hostTable.getColumn(MARS.SVC[i])
		.setCellRenderer(statusColumnRenderer);
	    hostTable.getColumn(MARS.SVC[i])
		.setPreferredWidth(STATWIDTH);
	}
  
	/* Create a cell renderer for SPOTS to gray out machines for which
	   SPOTS will not be probed. */
	DefaultTableCellRenderer spotsColumnRenderer = 
	    new DefaultTableCellRenderer() {
		public void setValue(Object value) {
		    /* Possibly pointless validation. Make sure the cell is a
		       string. If it's not a string, everything's gone right
		       to hell. :) */
		    String cellValue = (value instanceof String) ? 
		                       (String)value : "";
		    if (cellValue.length() <= 0)
		    setBackground(Color.lightGray);  // gray out inactive spots
		    else if (cellValue.equals(Host.TO))
		    setBackground(Color.yellow);     // yellow for dumb spots
		    else if (cellValue.equals(Host.FAIL))
		    setBackground(Color.red);        // red for dead spots
		    else
		    setBackground(Color.white);      // otherwise : white bg.
		    setText(cellValue);              // now pass the value
		}
	    };
	
	/* Now get each SPOTS column and set its cell renderer and width */
	for (int i=0; i < MARS.SPOT_COUNT; i++) {
	    hostTable.getColumn(MARS.SPOT[i])
		.setCellRenderer(spotsColumnRenderer);
	    hostTable.getColumn(MARS.SPOT[i])
		.setPreferredWidth(SPOTWIDTH);
	}
  
	hostTable.getColumn("Host").setPreferredWidth(HOSTWIDTH);
	
	/* more wizardry:
	   enable only cell selection, and set up the click listener for the 
	   table to be this UI. */
	hostTable.setCellSelectionEnabled(true);
	hostTable.setColumnSelectionAllowed(false);
	hostTable.setRowSelectionAllowed(false);
	hostTable.getSelectionModel().addListSelectionListener(this);
	
	/* end of heavy wizardry */
	
	/* Add the table to the scroll pane */
	scrollPane = new JScrollPane(hostTable);
	
	/* Create the status bar with a welcome message */
	statusBar = new JLabel("Welcome to MARS "+MARS.VERSION);
	
	/* Set the layout manager */
	this.getContentPane().setLayout(new BorderLayout());
	
	/* Adds the three critical items to the box */
	this.getContentPane().add(scrollPane,BorderLayout.CENTER);
	this.getContentPane().add(statusBar,BorderLayout.SOUTH);
	
	/* Set the window size. This number was determined on a Linux
	   machine. Your width may vary. :) */
	this.setSize(550,250);
	
	currentUI = this;
	
    }
    
    /** 
	Determine whether the system is running (default) or stopped
	(via the Control menu)

	@return true if the system is not stopped
    */
    public boolean returnRunningStatus() {
	return running; 
    }
    
    /**
       Return a reference to the current UI. Used to tell warning dialogs,
       etc. where to display, and for dispatching events
       
       @return the current MARSMainUI.
    */
    
    public static MARSMainUI getCurrent() {
	return currentUI;
    }

    /**
       Stops the system by shutting down all the probe threads.
    */
    public void stopSelected() {
	/* Stop MARS */
	MARS.stopThreads();
	/* Clear the probe thread list. */
	ProbeThread.clearAll();
	
	/* Change what the Control>Stop item looks like */
	stopItem.setEnabled(false);
	/* Change what the Control>Start item looks like */
	startItem.setEnabled(true);
	/* Clear the running flag */
	running = false;
    }

    /**
       Restarts the system by recreating all the probe threads.
    */
    public void startSelected() {
	/* Reinitialize the ProbeThreads */
	MARS.initProbes();
	/* Restart all the threads */
	MARS.startThreads();
	
	/* Change what the resume button looks like */
	startItem.setEnabled(false);
	/* Change what the pause button looks like */
	stopItem.setEnabled(true);
	/* Set the running flag */
	running = true;
    }


    /**
       Spawn the configuration interface, if none is active.
    */
    public void configSelected() {
	if (MARSConfigUI.getCurrent() == null)
	    new MARSConfigUI().setVisible(true);
    }
    
    /**
       Shut MARS down.
     */
    public void quitSelected()
    {
	MARS.shutdown();
    }

    /**
       Displays an alert notifying the user that a SPOTS parameter went out of
       bounds.

       @param parameter String specifying which parameter is out of bounds.
       @param host Name of host
       @param value Value of the offending parameter (String form)
       @param frame Frame to display this dialog over.
    */

    public static void DisplayBoundsAlert (String parameter, String host,
					   String value, JFrame frame) {
	/* The warning message */
	String outputMessage = parameter+" of "+value+" is out of bounds on "+
	    host;
	/* The top of the message box*/
	String topMessage = "WARNING: "+parameter+" on "+host;
	
	// Check for mail notification; send it if so
	if (MARSConfig.getCurrent().getNotifyAddr() != null)
	if (!MARSNotifier.sendParameterAlert(parameter, host, value))
	    // show warning message if sendServiceDown failed.
	    JOptionPane.showMessageDialog(frame,"Could not send notification."+
	       "\nThe notification mail server may be down.",
	       "Error sending notification.",JOptionPane.WARNING_MESSAGE);
	
	
	JOptionPane.showMessageDialog(frame,outputMessage,topMessage,
				      JOptionPane.WARNING_MESSAGE);
    }
    /**
       Displays an alert notifying the user that disk space is low on one (or
       more) filesystems.

       @param host Name of host
       @param fsList Space-delimited list of filesystems that are low on space
       @param frame Frame to display this dialog over.
    */

    public static void DisplayDiskAlert (String host,
					   String fsList, JFrame frame) {
	/* The warning message */
	String outputMessage = "Disk space is low on the following "+
	                       "filesystem(s) on "+host+":\n"+
	                       fsList;
	/* The top of the message box*/
	String topMessage = "WARNING: disk space on "+host;
	
	// Check for mail notification; send it if so
	if (MARSConfig.getCurrent().getNotifyAddr() != null)
	if (!MARSNotifier.sendDiskAlert(host, fsList))
	    // show warning message if sendServiceDown failed.
	    JOptionPane.showMessageDialog(frame,"Could not send notification."+
	       "\nThe notification mail server may be down.",
	       "Error sending notification.",JOptionPane.WARNING_MESSAGE);
	
	
	JOptionPane.showMessageDialog(frame,outputMessage,topMessage,
				      JOptionPane.WARNING_MESSAGE);
    }
    
    /**
       Display an alert notifying the user that a service on a specified host
       is down. 

       @param service String specifying which service is down.
       @param host Name of host
       @param frame Frame to display this dialog over.
    */
    
    public static void DisplayDownAlert (String service, String host,
					 JFrame frame) {
	/* The warning message */
	String outputMessage = "The "+service+" service is down on "+host+".";
	/* The top of the message box */
	String topMessage = "WARNING: "+service+" on "+host;
	
	// Check for mail notification; send it if so
	if (MARSConfig.getCurrent().getNotifyAddr() != null)
	if (!MARSNotifier.sendServiceDown(service, host))
	    // show warning message if sendServiceDown failed.
	    JOptionPane.showMessageDialog(frame,"Could not send notification."+
		  "\nThe notification mail server may be down.",
		  "Error sending notification.",JOptionPane.WARNING_MESSAGE);

	JOptionPane.showMessageDialog(frame,outputMessage,topMessage,
				      JOptionPane.WARNING_MESSAGE);
    }
    
    /** 
	Displays an info box informing the user what the server's 
	last response was.
	
	@param service the service number of the requested service
	@param host the host number of the specified host
	@param frame Frame to display this dialog over.
    */
    
    public static void DisplayLastResponse (int service, int host, 
					    JFrame frame) {
	
	// Get the service name from the service array
	String svcName = MARS.SVC[service];
	
	// Get host information from the central host table
	Host thisHost = MARSConfig.getCurrent().getHost(host);
	Probe thisProbe = thisHost.getSvcProbe(service);
	String hostName = thisHost.getVisibleName();
	
	// Abort now if service is disabled.
	if (thisProbe == null) 
	    return;
	
	// Compose an information box title
	String title = svcName+" on "+hostName;
	
	// Compose a last response header
	String header = "The last response from the "+svcName+" service on "
	    +hostName+" was:\n";
	
	// Get the last response.
	
	String lastResponse = thisProbe.getLastResponse();
	
	// Handle zero length.
	if (lastResponse.length() <= 0) 
	    lastResponse = "[No response]";
	
	// Display the information box
	JOptionPane.showMessageDialog(frame,header+lastResponse,title,
				      JOptionPane.PLAIN_MESSAGE);
    }

    /**
       Displays a dialog box containing a per-filesystem free space
       report for a specified host.
    
       @param host the host's number
       @param frame Frame to display this dialog over
    */
    
    public static void DisplayFSInfo (int host, JFrame frame) {

	// Get filesystem info from the host table
	Host thisHost = MARSConfig.getCurrent().getHost(host);
	String hostName = thisHost.getVisibleName();
	Hashtable fsinfo = thisHost.getFSInfo();
	// Fix NPE: if no host filesystem info is available, abort.
	if (fsinfo == null)
	    return;
	int fscount = fsinfo.size();

	// Build a JTable containing the filesystem info.
	// Define the column names
	String[] colhdr = new String[2];
	colhdr[0] = "Filesystem";
	colhdr[1] = "Free Space";	
	// Define a string array to hold the FS data
	String[][] data = new String[fscount][2];
	// Enumerate the fsinfo keys
	Enumeration filesystems = fsinfo.keys();
	// Iterate over the FSInfo keys, moving data from the hashtable
	// into the string array.
	int i = 0;                     // init string array counter
	while (filesystems.hasMoreElements()) {
	    // get the key
	    String key = (String) filesystems.nextElement();
	    // store it as the fs name
	    data[i][0] = key;
	    // get the FS free space
	    data[i][1] = (String) fsinfo.get(key);
	    // increment counter
	    ++i;
	}

	// Now create the table
	JTable fsinfoTable = new JTable(data,colhdr);

	// Add them both to a Box
	Box fsinfoBox = Box.createVerticalBox();	
	//	fsinfoBox.add(fsinfoLabel);
	fsinfoBox.add(fsinfoTable);
	    
	// Display the whole thing in a JOptionPane
	JOptionPane.showMessageDialog(frame,fsinfoTable,
				      "Free space per filesystem on "+hostName,
				      JOptionPane.PLAIN_MESSAGE);
    }
	    
    /**
       Changes the message displayed in the status bar
    
       @param newStatus the new string to display
    */
    
    public void setStatus(String newStatus) {
	statusBar.setText(newStatus);
    }
    
    /**
       Handles a ListSelectionEvent for the main table. This code dispatches
       mouse clicks in the main table, and will one day be hooked up to detail
       views and configuration options.
       
       @param lse a ListSelectionEvent generated by a user click on the table
    */
    public void valueChanged(ListSelectionEvent lse) {
	if ((!lse.getValueIsAdjusting()) ||
	    hostTable.getSelectionModel().isSelectionEmpty())
	    return;                    // makes each click register only once
	                               // does not register empty table events
	int row = hostTable.getSelectedRow();   
	int col = hostTable.getSelectedColumn();

	if (DEBUG) 
	    System.out.println("MARSMainUI.vc(): click at ("+row+","+col+")");
	
	if ((col > 0) && (col < (MARS.SVC_COUNT + 1))) {// this is a svc column
	    // display last response over the current UI
	    if (DEBUG)
		System.out.println("MARSMainUI.vc(): dispatch lastResponse");
	    MARSMainUI.DisplayLastResponse(col-1, row, getCurrent());
	} else if (col == MARS.SVC_COUNT + MARS.SPOT_DISK + 1) {
	    // this is the disk column
	    // display a per-filesystem report over the current UI.
	    if (DEBUG)
		System.out.println("MARSMainUI.vc(): dispatch fsInfo");
	    MARSMainUI.DisplayFSInfo(row, getCurrent());
	}
	// unselect the current table selection
	// this is an event monitor, after all, not a table selector
	hostTable.getSelectionModel().clearSelection();
	
    }
    
} // end class MARSMainUI












