/*
  Copyright 1999 Brian H. Trammell (brian@altara.org)
  Copyright 1999 Ben Parrott (benji@altara.org)
  
  This software (and its source code) are freely usable, modifiable, 
  and redistributable under the terms of the GNU General Public License, 
  available at http://www.gnu.org/copyleft/gpl.html, and in the ./COPYING 
  file in the MARS distribution.
*/

package org.altara.mars;

import java.net.*;
import java.io.*;

/**
   Probe is the lowest level of MARS; it contains the networking code used
   to probe remote hosts. This class is abstract, as it does not know how
   to speak any specific protocol. There is one subclass of Probe for each
   service MARS can probe. 
*/

// Revision 0.1 : 25 Jan 1999 bht - initial entry
// Revision 0.3 : 07 Feb 1999 bht - fixed javadoc for design doc
// Revision 1.2.0 : 21 Mar 1999 bht - fixed deep cloning bug
// Revision 1.4.1 : 26 Apr 1999 bht - added getLastResponse()
// Revision 1.4.4 : 11 May 1999 bht - fixed the Bowers bug

public abstract class Probe implements Serializable {

    /** Compile-time debugging switch. Should be false. */
    public static final boolean DEBUG = false;

  // Instance variables
  Host host;                      // Host to probe
  int port;                       // Port to probe
  String send;                    // String to send (optional)
  String expect;                  // String to expect
  long timeout = 3000;            // msec to wait for success
  StringBuffer probeInBuf;        // allows all methods access
  
  // Constructors
  
  /**
     Creates a probe on a specified port on a specified host, that will
     send nothing on connection and expect a specified string.
     
     @param host The Host to probe
     @param port The port to probe
     @param expect The string to expect upon opening the connection
     @return a new Probe with the specified properties
  */

  protected Probe (Host host, int port, String expect) {
    this.host = host;
    this.port = port;
    this.expect = expect;
    if (DEBUG) System.out.println("mars.Probe(): new expect instance.");
  }

  
  /**
     Creates a probe on aspecified port on a specified host, that will
     send a specified string on connection and expect a specified string
     in return.
     
     @param host The Host to probe
     @param port The port to probe
     @param send The string to send upon opening the connection
     @param expect The string to expect in reply
     @return a new Probe with the specified properties
  */

  protected Probe (Host host, int port, String send, String expect) {
    this.host = host;
    this.port = port;
    this.send = send;
    this.expect = expect;
    if (DEBUG) System.out.println("mars.Probe(): new send/expect instance.");
  }

  // Action methods

  /**
     Probes the host, returns the result. The results should be interpreted
     as follows: true means the probe is OK (set status to Host.OK), false
     means the probe has timed out (set status to Host.TO), and if an
     exception is thrown, the probe failed outright (set status to Host.FAIL)

     @return true if the probe succeeded, false if not.
     @exception UnknownHostException if it couldn't find the target host
     @exception IOException if there was an error reading from the socket
     @exception InterruptedException if the timeout wait was interrupted
  */

  public boolean getStatus() throws UnknownHostException, IOException, InterruptedException {
    
    Socket probeSock;               /* Socket used in probing */
    InputStreamReader probeIn;
    OutputStreamWriter probeOut;
    int thisChar;

    // Get the hostname out of the Host
    String hostname = host.getDNSName();

    if (DEBUG) System.out.println("mars.Probe.gS(): starting probe");

    // Set the timeout value
    timeout = MARSConfig.getCurrent().getTimeout();

    // Create a buffer to read data into.
    // We create a new one early in the getStatus() process so "down"
    // connections correspond to empty buffers.
    probeInBuf = new StringBuffer();

    // Open the probe socket
    probeSock = new Socket(hostname,port);
    if (DEBUG) 
      System.out.println ("mars.Probe.gS(): opened socket "+hostname+":"+port);
 
    // Get and write to the output stream if we have to send something
    if (send != null) {
      if (DEBUG) System.out.println("mars.Probe.gS(): preparing to send...");
      probeOut = new OutputStreamWriter(probeSock.getOutputStream());
      probeOut.write(send,0,send.length());
      probeOut.flush();
      if (DEBUG) 
	System.out.println("mars.Probe.gS(): wrote "+send+" to socket");
    }
    
    // Now wait for the server to respond.
    if (DEBUG) System.out.println("mars.Probe.gS(): sleeping "+timeout+" ms");
    Thread.currentThread().sleep(timeout);

    // Get the input stream.
    if (DEBUG) System.out.println("mars.Probe.gS(): preparing to receive...");
    probeIn = new InputStreamReader(probeSock.getInputStream());
    probeInBuf = new StringBuffer();
    
    // Now keep reading data from the socket until there is no more.
    if (DEBUG) System.out.print("mars.Probe.gS(): reading into buffer: ");
    while (probeIn.ready()) {
      thisChar = probeIn.read();
      probeInBuf.append((char)thisChar);
      if (DEBUG) System.out.print((char)thisChar);
    }
    if (DEBUG) System.out.println(" done.");

    // Close the socket
    probeSock.close();

    // Check for the expected string somewhere within the one we got...
    if (probeInBuf.toString().indexOf(expect) >= 0) {
      return true;
    } else {
      return false;
    }
  }

  /**
     Return the last string the service responded with. This is so users can
     see what happened to a "timed out" service.

     @return the last string recieved from the service
  */
  
  public String getLastResponse() {
      return probeInBuf.toString();
  }

}





